#include "desktopbackground.h"

#include <QSGSimpleRectNode>
#include <QGSettings/QGSettings>
#include <QPainter>


QT_BEGIN_NAMESPACE
extern Q_DECL_EXPORT void qt_blurImage(QPainter *p, QImage &blurImage, qreal radius, bool quality, bool alphaOnly, int transposed = 0);
QT_END_NAMESPACE

//! 默认的最大虚化程度
#define MAX_BLUR_ALPHA  100
//! 背景文件获取失败时，默认加载的背景
#define DEFAULT_BACKGROUND   ":/images/default-background.jpg"

DesktopBackground::DesktopBackground(QQuickPaintedItem *parent)
    : QQuickPaintedItem(parent)
    , m_blurAlpha(0)
{
    setFlag(ItemHasContents);

    init();
}

void DesktopBackground::init()
{
    connect(this, &DesktopBackground::blurAlphaChanged, this, &DesktopBackground::onBlurAlphaChanged);

    m_backgroundFile = getBackgroundFile();
    if(m_backgroundFile == ""){
        qWarning() << "未获取到背景图片!";
        m_backgroundFile = DEFAULT_BACKGROUND;
    }
    m_background = QImage(m_backgroundFile);
    m_blurBackground = m_background;

    setBlurAlpha(0);
}

void DesktopBackground::setBlurAlpha(qreal alpha)
{
    if(alpha  == m_blurAlpha || alpha < 0 || alpha > 1)
        return;
    m_blurAlpha = alpha;
    emit blurAlphaChanged();
}


void DesktopBackground::paint(QPainter *painter)
{
    painter->setBrush(Qt::black);
    painter->drawImage(boundingRect(), m_blurBackground);
}

QString DesktopBackground::getBackgroundFile()
{
    QUrl url;
    QString filePath;
    QString schemaUrl = "org.mate.background";
    QString keyName =   "pictureFilename";

    if (QGSettings::isSchemaInstalled(schemaUrl.toUtf8())) {
        QGSettings gsettings(schemaUrl.toUtf8());

        const QStringList keyList = gsettings.keys();
        if (keyList.contains(keyName)) {
            QString name = gsettings.get(keyName).toString();
            url = QUrl::fromLocalFile(name);
            filePath = url.toString().replace(QRegExp("^file:/"), "");
        } else {
            qWarning() << QString("未获取到Gesetting的键值(%1) value").arg(keyName);
            return "";
        }
    }

    return filePath;
}

void DesktopBackground::onBlurAlphaChanged()
{
    if(m_background.isNull())
        return;
    m_blurBackground = m_background;

    QPainter painter(&m_blurBackground);
    qt_blurImage(&painter, m_background, m_blurAlpha * MAX_BLUR_ALPHA, true, false);
    update();
}
