#ifndef MULTITASKVIEWMANAGER_H
#define MULTITASKVIEWMANAGER_H

#include <memory>

#include <QQuickView>

#include <ukui-kwin/kwineffects.h>

#include "dbusservice.h"
#include "multitaskviewmodel.h"

namespace MultitaskView {

class MultitaskViewManager : public KWin::Effect
{
    Q_OBJECT

public:

    MultitaskViewManager();

    ~MultitaskViewManager() override;

    void grabbedKeyboardEvent(QKeyEvent* e) override;

    Q_INVOKABLE void open();

    Q_INVOKABLE void close();

private:

    void initModel();

    void registerQmlTypes();

    void registerShortcut();

    void initDBusService();

    void initView();

    void destroyView();

    void connectSignals();

private:

    std::unique_ptr<MultitaskViewModel> model_ = nullptr;

    std::unique_ptr<QQuickView> view_ = nullptr;

    std::unique_ptr<DBusService> dbusService_ = nullptr;

    bool isOpen = false;
};

} // namespace MultitaskView

#endif // MULTITASKVIEWMANAGER_H
