import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "Controller.js" as Controller
import IconTheme 1.0

Rectangle {
    id : previewWindow
    width : 100
    height : 150
    color : "transparent"

    /** windowType is the type of PreviewWindow,
    * the options are : none, app, desktop, creation.
    * Default to none.
    */
    property string windowType : "none"

    /** stateList is the list of state,
    * the options are : default, hover, drag, receive.
    * Default to default.
    */
    property var stateList : ["default", "hover", "drag", "receive"];
    state : "default"
    property int wid : -1
    property int previewAreaWidth : 100
    property int previewAreaHeight : 150
    property real posX : 0
    property real posY : 0
    property double closeButtonSize : 24
    property bool closeButtonVisble : false
    property double titleOpacity : 1
    property var iconVisble : true
    property var controller : null
    property real borderWidth : 0
    property var dragAxis : Drag.XAndYAxis
    property var captionText  : ""
    property bool winowDragActive : previewMouseArea.drag.active
    property bool nowDragActive : false
    Drag.active : previewMouseArea.drag.active
    Drag.source : previewWindow
    Drag.hotSpot.x : previewWindow.previewAreaWidth
    Drag.hotSpot.y : previewWindow.previewAreaHeight

    onWinowDragActiveChanged : {
        if(winowDragActive !== nowDragActive){
            nowDragActive = winowDragActive
        }
    }

    NumberAnimation on x{
        id : movePreX
        duration : 500
        running : false
    }

    NumberAnimation on y{
        id : movePreY
        duration : 500
        running : false
    }

    function setStatus(status) {
        if(stateList.indexOf(status) < 0)
        {
            return;
        }
        if (status === state) {
            return
        } else {
            state = status;
        }
    }

    function setCaption(titleText) {
        caption.text = titleText;
    }

    function setId(wId) {
        wid = wId;
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function setDragStatus() {
        setStatus("drag")
    }

    function setReceiveStatus() {
        setStatus("receive")
    }

    function returnOriginal(x,y)
    {
        movePreX.to =0
        movePreY.to =0
        movePreX.from =x
        movePreY.from =y
        movePreX.stop()
        movePreY.stop()
        movePreX.start()
        movePreY.start()
    }

    // 应用程序预览窗口的缩放动画暂时
    // 由视图模块在响应拖拽事件时主动
    // 触发。因此，该函数暂时实现为空
    function onEnterDropArea() {

    }

    // 应用程序预览窗口的缩放动画暂时
    // 由视图模块在响应拖拽事件时主动
    // 触发。因此，该函数暂时实现为空
    function onExitDropArea() {

    }

    function zoomIn() {
        scaleAnimation.running = false
        scaleAnimation.stop();
        scaleAnimation.from = 1;
        scaleAnimation.to = 0.25;
        scaleAnimation.start();
    }

    function zoomOut() {
        scaleAnimation.running = false
        scaleAnimation.stop();
        scaleAnimation.from = 0.5;
        scaleAnimation.to = 1;
        scaleAnimation.start();
    }

    NumberAnimation on scale{
        id : scaleAnimation
        duration : 500
        from : 0
        to : 0
        running : false
    }

    MouseArea{
        id : previewMouseArea
        anchors.fill : parent
        hoverEnabled : true
        preventStealing : true
        drag.target : previewWindow
        drag.axis : dragAxis

        onEntered : {
            if(!controller)
            {
                return;
            }
            controller.setHoverStatus();
        }

        onExited : {
            if(!controller)
            {
                return;
            }
            controller.setDefaultStatus();
        }


        onClicked : {
            if(!controller)
            {
                return;
            }
            controller.onWindowClicked(wid);
        }

        onReleased : {
            previewWindow.Drag.drop();
        }
    }


    ColumnLayout {
        spacing : 0
        anchors.centerIn : parent
        Layout.fillHeight : true
        width : parent.width

        RowLayout {
            id : captionRow
            spacing : 8
            Layout.fillHeight : true
            opacity : titleOpacity

            Rectangle {
                id : iconItem
                height : 24
                width : 24
                color : "transparent"
                visible : iconVisble
                IconTheme{
                    anchors.fill : parent
                    icon : multitaskViewModel.getWindowIcon(wid)
                }
            }

            Label {
                id : caption
                Layout.fillWidth : true
                text : captionText
                elide : Text.ElideRight
                color : "#FFFFFF"
            }

            Rectangle {
                id : closeButton
                visible : closeButtonVisble
                height : 24
                width : 24
                radius : 12
                color : Qt.rgba(255/255, 255/255, 255/255, 0.15)

                IconTheme{
                    anchors.centerIn : parent
                    width : 12
                    height : 12
                    iconName : "window-close-symbolic"
                }

                MouseArea{
                    anchors.fill : parent

                    onClicked : {
                        if(!controller)
                        {
                            return;
                        }
                        controller.removePreviewWindow(wid);
                    }
                }
            }
        }//RowLayout

        Rectangle {
            id : previewArea
            width : previewAreaWidth
            height : previewAreaHeight
            color : "transparent"
            border.color : "#FFFFFF"
            border.width : borderWidth
            radius : 12
        }

        states : State {
            when : previewMouseArea.drag.active
            ParentChange { target : previewWindow; parent : root }
            PropertyChanges { target : previewWindow; borderWidth : 4 }
            PropertyChanges { target : previewWindow; titleOpacity : 0 }
        }
    }//culumnLayout
}
